#ifndef bqtRockEditATOMsHH
#define bqtRockEditATOMsHH

#include <string>

struct Char
{
    unsigned char tile;    //0-FF
    unsigned char palette; //0-3
public:
    bool operator== (const Char& b) const
    {
        return tile==b.tile && palette==b.palette;
    }
};

struct Block
{
    Char c[4];
public:
    void Write(unsigned offs, unsigned color_offs) const;
    void Load(unsigned offs, unsigned color_offs);
    
    bool operator==(const Block& b) const
    {
        return c[0] == b.c[0]
            && c[1] == b.c[1]
            && c[2] == b.c[2]
            && c[3] == b.c[3];
    }
};

struct Room
{
    unsigned char blocks[8*8]; // blockid
public:
    void Write(unsigned ptroffs, unsigned offs) const;
    void Load(unsigned ptroffs);
    
    bool operator==(const Room& b) const
    {
        for(unsigned a=0; a<64; ++a)
            if(blocks[a] != b.blocks[a])
                return false;
        return true;
    }
};

struct Active
{
    unsigned char type;
    unsigned char screen;
    unsigned char x1,y1, x2,y2;
    unsigned char counter; // used for ordering buzz-blocks
    
    Active() : screen(255), counter(0) { }
    Active(int t,int s,int x,int y,int X,int Y):
      type(t),screen(s),x1(x),y1(y),x2(X),y2(Y), counter(0) { }
 
    void Delete() { screen=255; }
 
    bool operator< (const Active& b) const
    {
        if(screen != b.screen) return screen < b.screen;
        if(x1 != b.x1) return x1 < b.x1;
        if(type != b.type) return type < b.type;
        if(counter != b.counter) return counter < b.counter;
        return x2 < b.x2;
    }
};

struct Enemy
{
    unsigned char screen;
    unsigned char x,y;
    unsigned char type;
    
    Enemy() : screen(255) { }
    Enemy(int s,int X,int Y,int t):
      screen(s),x(X),y(Y),type(t) { }

    void Delete() { screen=255; }
 
    bool operator< (const Enemy& b) const
    {
        if(screen != b.screen) return screen < b.screen;
        if(x != b.x) return x < b.x;
        return y < b.y;
    }
};

struct Screen
{
    char ascii[32*16+1];
    unsigned MonsterGraphicsIndex; // Index into MonsterGraphicsData
    int cont_dir;

    Char outmap[16*16];
};

struct MonsterGraphicsData
{
    unsigned char index[5];
    unsigned char palette[6];
    
    const std::string to_string() const
    {
        return std::string((const char*)index, (const char*)(index+5))
             + std::string((const char*)palette, (const char*)(palette+6));
    }
};
struct LevelGraphicsData
{
    unsigned char index;
    unsigned char count; // 0 = not gfx.
};

#endif
