static void ElecmanAscii(Screen& s)
{
    GenericAsciiMaker(s,
// Sky
-1,'.', 0x01,2,
// Void
-1,',', 0x00,0,
// Background
-1,' ', 0x24,3, // branch: wire1 up, wire2 left, wire3 right
        0x25,3, // horizontal wire3
        0x26,3, // branch: wire3 left, wire2 right, wire1 down
        0x27,3, // horizontal wire2
        0x2B,3, // vertical wire1
        0x2C,3, // vertical wire3
        0x2D,3, // decoration, wants wire3 down, can take wires left
        0x2E,3, // decoration, wants wire2 right, can take wires up
        0x2F,3, // background
        0x34,3, // background shadow
        0x36,3, 0x37,3,
        0x3E,3, 0x3F,3, // background detail (inset)
// Walls
-1,'#', 0x60,0, 0x61,0, 0x62,0,
        0x68,0, 0x69,0, 0x6A,0, // wall
-1,'#', 0x70,0, // single block platform
        0x60,0, 0x61,0, 0x62,0, // longer platform (edges used only if not anchored in wall)
-1,'#', 0x72,0, // a decoration inside the wall, randomly appearing

-1,'A', 0x73,1, // arc generator
-1,'$', 0x7C,1,0x7D,1, // arcbox (wall decoration)
-1,'R', 0x75,0, // one type of vertical wall (or horizontal!)
-1,'R', 0x71,0, // another type of vertical wall
-1,'#', 0x51,0, 0x52,0, 0x53,0, // Wily signature
// Ladder
-1,'|', 0x83,0,
// Wily door
-1,'D', 0x10,0,
// Gutsblock
-1,'G', 0x38,0, 0x39,0,
        0x3A,0, 0x3B,0,
// no spikes
-2);
}
#include "legofill.hh"

static bool ElecAir(char c)
{
    return c==' ' || c==',' || c=='.';
}
static bool ElecWall(char c)
{
    return c=='R' || c=='#' || c=='A' || c=='$';
}

static void ElecmanChar(Screen& s)
{
    unsigned randseed = 0;
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
            randseed += s.ascii[y*32+x];
    std::srand(randseed);
    
    unsigned char decorations[16*16] = {0};
    
    unsigned char Lego1[2][3] = {{0x60,0x61,0x62},{0x68,0x69,0x6A}};
    
    LegoFill(decorations, s.ascii,'#', Lego1,Lego1);
    
    // Find single-block-height platforms.
    for(unsigned y=1; y<15; ++y) 
        for(unsigned x=0; x<16; ++x)
        {
            if(s.ascii[(y)  *32+x]=='#'
            && ElecAir(s.ascii[(y-1)*32+x])
            && ElecAir(s.ascii[(y+1)*32+x]))
            {
                // platform that is wholly in air!
                
                // Find the left edge
                unsigned begin = x;
                while(begin > 0)
                {
                    unsigned testx = begin-1;
                    bool air = ElecAir(s.ascii[(y-1)*32+testx]);
                    bool wall=!ElecAir(s.ascii[(y  )*32+testx]);
                    if(!wall || !air) break;
                    begin=testx;
                }
                // Find the right edge
                unsigned end = x;
                while(end < 15)
                {
                    unsigned testx = end+1;
                    bool air = ElecAir(s.ascii[(y-1)*32+testx]);
                    bool wall=!ElecAir(s.ascii[(y  )*32+testx]);
                    if(!wall || !air) break;
                    end=testx;
                }
                unsigned length = end-begin+1;
				if(length == 1)
					decorations[y*16+begin] = 0x70; // single block platform
				else
				{
				    bool anchored = begin==0 || !ElecAir(s.ascii[y*32+begin-1]);
					decorations[y*16+begin] = anchored ? 0x61 : 0x62;
		        }
			    for(unsigned xpos=begin+1; xpos<end; ++xpos)
			        decorations[y*16+xpos] = 0x61; // platform
				if(length != 1)
				{
				    bool anchored = end==15 || !ElecAir(s.ascii[y*32+end+1]);
					decorations[y*16+end] = anchored ? 0x61 : 0x62;
		        }
		        x = end;
            }
        }
    
    // Convert to tiles
    for(unsigned y=0; y<16; ++y) 
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c   = 0;
            unsigned char pal = 0;
            
            switch(s.ascii[y*32+x])
            {
                case '.':
                {
                    // background
                    pal = 2;
                    c   = 0x01;
                    break;
                }
                case ',':
                {
                    // void
                    pal = c = 0;
                    break;
                }
                case ' ':
                {
                    // background
                    pal = 3;
                    c   = 0x2F;
                    break;
                }
                case '#':
                {
                    char deco = decorations[y*16+x];
                    pal = 0;
                    c   = deco ? deco : 0x70;
                    break;
                }
                case 'A':
                    c = 0x73;
                    pal = 1;
                    break;
                case '$':
                    pal = 1;
                    c = 0x7C + (x&1);
                    break;
                case 'R':
                {
                    bool right = x<15 && (s.ascii[y*32+(x+1)]!='R');
                    bool left  = x>0  && (s.ascii[y*32+(x-1)]!='R');
                    pal = 0;
                    c = (right||left) ? 0x71 : 0x75;
                    // 0x71 is when there's air next to it.
                    break;
                }
                case Ascii_Ladder:
                    c   = 0x83; // ladder
                    pal = 0x00;
                    break;
                case Ascii_Spike:
                    c   = 0xC4; // spike
                    pal = 0x01;
                    break;
                case Ascii_WilyDoor:
                    c   = 0x10;
                    pal = 0x00;
                    break;
                case Ascii_Guts:
                    c   = 0x38 + (x&1) + (y&1)*2; // gutsblock
                    pal = 0x00;
                    break;
                default:
                    printf("? '%c'\n", s.ascii[y*32+x]);
            }
            
            s.outmap[y*16+x].tile    = c;
            s.outmap[y*16+x].palette = pal;
        }
}

static Level Elecman =
{
	4, // level number
	"Elecman",

	// screens:
	{
		{
		"RR           |RR"   "                "
		"RR           |RR"   "                "
		"RR           |RR"   "                "
		"RR           |RR"   "                "
		"RR        ####RR"   "                "
		"RR            RR"   "  f             "
		"RR#####       RR"   "                "
		"RR            RR"   "             f  "
		"RR        ####RR"   "                "
		"RR            RR"   "      f         "
		"RR#####       RR"   "                "
		"RR            RR"   "                "
		"RR            RR"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Up| LevelBegin},
		{
		"RR |          RR"   "                "
		"RR |          RR"   "                "
		"RR |          RR"   "                "
		"RR |          RR"   "                "
		"RR |          RR"   "                "
		"RR ##         RR"   "             E  "
		"RR ##       ##RR"   "                "
		"RR ##         RR"   "     f          "
		"RR ###A       RR"   "                "
		"RR            RR"   "         f      "
		"RR      #A    RR"   "                "
		"RR            RR"   "                "
		"RR            RR"   "                "
		"###|#  ######|RR"   "                "
		"###|#  ######|RR"   "                "
		"###|#  ######|RR"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"RR |          RR"   "                "
		"RR |          RR"   "                "
		"RR |          RR"   "                "
		"RR |          RR"   "                "
		"RR |          RR"   "                "
		"RR ##         RR"   "                "
		"RR ##       ##RR"   "                "
		"RR ##         RR"   "                "
		"RR ###A       RR"   "         -      "
		"RR            RR"   "                "
		"RR      #A    RR"   "            =   "
		"RR            RR"   "                "
		"RR            RR"   "                "
		"###|#  ######|RR"   "                "
		"###|#  ######|RR"   "                "
		"###|#  ######|RR"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR |###|      RR"   "                "
		"RR |###|      RR"   "                "
		"RR |###|      RR"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR     |      RR"   "                "
		"RR |###|      RR"   "                "
		"RR |###|      RR"   "                "
		"RR |###|      RR"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"RR |         |RR"   "                "
		"RR |         |RR"   "                "
		"RR |         |RR"   "                "
		"RR #         |RR"   "                "
		"RR           |RR"   "                "
		"RR           #RR"   "                "
		"RR            RR"   "                "
		"RR            RR"   "                "
		"RR            RR"   "                "
		"RR            RR"   "                "
		"RR       #    RR"   "                "
		"RR            RR"   "                "
		"RR            RR"   "                "
		"RR     |##    RR"   "                "
		"RR     |##    RR"   "                "
		"RR     |##    RR"   "                "
		, 1 // monster graphics index
		,Up},
		{
		"...|....########"   "                "
		"...|....########"   "                "
		"...|......######"   "                "
		"...|......##$$##"   "               "
		"...|......######"   "                "
		"...|......######"   "                "
		"#######|########"   "                "
		"##$$###|####$$##"   "               "
		"#######|  ######"   "                "
		"##$$###|  ######"   "                "
		"########      RR"   "                "
		"RR            RR"   "                "
		"RR           |RR"   "     ppp        "
		"RR |####     |RR"   "                "
		"RR |         |RR"   "                "
		"RR |         |RR"   "                "
		, 1 // monster graphics index
		,Up},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		".....###########"   "                "
		".....#$$####$$.."   "               "
		"...|##########.."   "               E"
		"...|##$$####$$##"   "               "
		"...|############"   "                "
		"...|############"   "                "
		, 0 // monster graphics index
		,RestartPointA| Right},
		{
		"............|..."   "                "
		"............|..."   "                "
		"............|..."   "                "
		"............|..."   "                "
		"................"   "                "
		"................"   "                "
		".............#.."   "                "
		"..........#....."   "                "
		"................"   "                "
		"................"   "                "
		"...........#...."   "                "
		"................"   "                "
		".........#......"   "                "
		"..#...#........."   "                "
		"................"   "                "
		"................"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"....|.##|     ##"   "                "
		"......        ##"   "                "
		"......        ##"   "                "
		"############|###"   "                "
		"############|###"   "                "
		"############|###"   "                "
		"############|###"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"...|....##  | ##"   "                "
		"...|....$$  | ##"   "                "
		"...|....##  | ##"   "                "
		"...|....##  | A#"   "  f         -   "
		"..##....##|#####"   "                "
		"........$$|#####"   "      f         "
		"......####|   ##"   "                "
		"........##|   A#"   "   f        =   "
		"..##....####| ##"   "                "
		"........$$##| ##"   "               "
		"......##    | ##"   "                "
		"......##    | ##"   "                "
		"..##|###|#######"   "                "
		"..##|###|#######"   "                "
		"..##|###|#######"   "                "
		"..##|###|#######"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"....|...##  | ##"   "                "
		"....|...##  | ##"   "                "
		"....|...##  | ##"   "                "
		"........$$  | ##"   "               "
		"........##  | ##"   "                "
		"........##  | ##"   "                "
		"......####    ##"   "                "
		"........$$    ##"   "                "
		"..##....GG    ##"   "                "
		"........GG    ##"   "                "
		"......######|###"   "                "
		"......##$$  | ##"   "               "
		"..#|######  | ##"   "                "
		"..#|######  | ##"   "                "
		"..#|######  | ##"   "                "
		"..#|######  | ##"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"...|........|.##"   "                "
		"...|........|.##"   "                "
		"...|........|.##"   "                "
		"...|........|.##"   "                "
		"......##########"   "                "
		"..##..##########"   "                "
		"......GGGGGG..##"   "                "
		"......GGGGGG..##"   "             M  "
		"..|#############"   "                "
		"..|#############"   "                "
		"..|...........##"   "                "
		"..|...........##"   "                "
		"..##|#######|###"   "                "
		"..##|#######|###"   "                "
		"..##|#######|###"   "                "
		"..##|#######|###"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"....|...##  | ##"   "                "
		"....|...##  | ##"   "                "
		"....|...##  | ##"   "                "
		"....|...$$  | ##"   "               "
		"..##|...##  | ##"   "                "
		"........##  | A#"   "      f     -   "
		"......####  | ##"   "                "
		"........$$  | ##"   "   f            "
		"..|###..GG  | ##"   "                "
		"..|.....GG  | A#"   "            =   "
		"..|.......##| ##"   "                "
		"..|.......##| ##"   "      f         "
		"###|..######|###"   "                "
		"###|########|###"   "                "
		"###|########|###"   "                "
		"###|########|###"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"..|..#####  | ##"   "                "
		"..|..#####  | ##"   "                "
		"..|..#####  | ##"   "                "
		"..|..###$$  | A#"   "            -   "
		"..|..#####  | ##"   "                "
		"..|..A####  | ##"   "   -            "
		"..|..##     | ##"   "                "
		"..|..##     | ##"   "                "
		"..|..##|########"   "                "
		"..|..A#|########"   "   =            "
		"..|..##|      ##"   "                "
		"..|..##|      ##"   "                "
		"..##|#######|###"   "                "
		"..##|#######|###"   "                "
		"..##|#######|###"   "                "
		"..##|#######|###"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"...|......######"   "                "
		"...|......######"   "                "
		"...|..........##"   "                "
		"...|..........##"   "                "
		"...|..######..##"   "                "
		"...|..######..##"   "             j  "
		"......##    GG##"   "                "
		"......A#    GG##"   "    -    j      "
		"..|#####|#######"   "                "
		"..|#####|#######"   "                "
		"..|...##|     ##"   "                "
		"..|...A#|     ##"   "    =           "
		"..|...######|###"   "                "
		"..|...######|###"   "                "
		"..|...######|###"   "                "
		"..|...######|###"   "                "
		, 2 // monster graphics index
		,Up},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		".....###########"   "                "
		".....#$$####$$.."   "                "
		"...|##########.."   "                "
		"...|##$$####$$##"   "              "
		"...|############"   "                "
		"...|############"   "                "
		, 3 // monster graphics index
		,Right},
		{
		"............|..."   "                "
		"............|..."   "                "
		"............|..."   "                "
		"............|..."   "                "
		"................"   "                "
		"................"   "                "
		".............#.."   "                "
		"..........#....."   "                "
		"................"   "                "
		"................"   "                "
		"...........#...."   "                "
		"................"   "                "
		".........#......"   "                "
		"..#...#........."   "                "
		"................"   "                "
		"................"   "                "
		, 3 // monster graphics index
		,Up},
		{
		"..|#############"   "                "
		"..|#########$$##"   "                "
		"..|.......######"   "                "
		"..|.......##$$##"   "               "
		"..|.......######"   "                "
		"..|.......##$$##"   "               "
		"............####"   "                "
		"............$$##"   "                "
		"...A..........##"   "      -         "
		"..............##"   "                "
		".....A........##"   "        =       "
		"..............##"   "                "
		"..............##"   "                "
		"############|.##"   "                "
		"############|.##"   "                "
		"############|.##"   "                "
		, 0 // monster graphics index
		,Up},
		{
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		"..............DD"   "              SS"
		"..............DD"   "              SS"
		"..............DD"   "           t  SS"
		"..............DD"   "              SS"
		"..........######"   "                "
		"..........##$$##"   "               "
		"..|#############"   "                "
		"..|#####$$##$$##"   "               "
		"..|#############"   "                "
		"..|#############"   "                "
		, 4 // monster graphics index
		,Shutter},
		{
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		"######  | ##,,,,"   "                "
		"######  | ##,,,,"   "                "
		"          ##,,,,"   "                "
		"          ##,,,,"   "                "
		"          ##,,,,"   "                "
		"          ##,,,,"   "                "
		"############,,,,"   "                "
		"############,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		, 0 // monster graphics index
		,Up| RestartPointB},
		{
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | A#,,,,"   "        -       "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | A#,,,,"   "        =       "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | A#,,,,"   "        -       "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		, 0 // monster graphics index
		,Up},
		{
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | A#,,,,"   "        -       "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | A#,,,,"   "        =       "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | A#,,,,"   "        -       "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		",,,,##  | ##,,,,"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"################"   "                "
		"################"   "                "
		"R              R"   "                "
		"R              R"   "                "
		"R              R"   "                "
		"R              R"   "                "
		"R              R"   "                "
		"R              R"   "                "
		"R              R"   "                "
		"R              R"   "                "
		"##GGGG         R"   "                "
		"##GGGG         R"   "                "
		"######  | ######"   "                "
		"######  | ######"   "                "
		"######  | ######"   "                "
		"######  | ######"   "                "
		, 5 // monster graphics index
		,End},
	},

	// palette
	{
		15, 32, 39, 23, 
		15, 32, 39, 33, 
		15, 15, 15, 12, 
		15, 40, 24,  8, 

		15, 44, 17, 15, 
		15, 32, 56, 15, 
		15, 48, 42, 15, 
		15, 48, 38, 15, 

		15, 32, 39, 23, 
		15, 32, 39, 33, 
		15, 15, 15, 12, 
		15, 32, 16,  0, 
	},

	// after doors palette
	{
		15, 38, 23,  7, 
		15, 48, 16,  2, 
		15, 38, 23,  2, 
		15, 55, 39, 23, 

	},

	// actives
	{
		Active( 0, 5, 0x40,0x50, 0x50,0x60 ),
		Active( 0, 5, 0x60,0x70, 0x70,0x80 ),
		Active( 0, 5, 0x90,0x70, 0xA0,0x80 ),
		Active( 0, 5, 0xB0,0x50, 0xC0,0x60 ),
		Active( 0, 6, 0x90,0xC0, 0xA0,0xD0 ),
		Active( 0, 6, 0xA0,0xC0, 0xB0,0xD0 ),
		Active( 0, 6, 0xB0,0xC0, 0xC0,0xD0 ),
		Active( 0, 6, 0xC0,0xC0, 0xD0,0xD0 ),
	},

	// enemies
	{
		Enemy( 3, 0x10,0xFF, 0x6 ),
		Enemy( 4, 0x10,0xFF, 0x6 ),
		Enemy( 5, 0x10,0x80, 0x10 ),
		Enemy( 6, 0x10,0x80, 0x10 ),
		Enemy( 9, 0x10,0xFF, 0x6 ),
		Enemy( 11, 0x10,0xFF, 0x6 ),
	},

	// level gfx (id, count) (max 16); id is block*4+bank
	{
		{ 0x40, 8 }, { 0x6E, 1 }, { 0x92, 1 }, { 0x96, 1 }, 
		{ 0x62, 1 }, { 0x72, 1 }, { 0xE2, 1 }, { 0xD8, 1 }, 
		{ 0x5E, 1 }, { 0xC0, 8 }, { 0xC1, 8 }, 
	},

	// monster gfx tables (max 11) (5 tiles, 2*3bytepalette)
	{
		{ { 0x6E, 0x92, 0x96, 0x66, 0x72, }, { 48,42,15, 48,38,15,} },
		{ { 0x8A, 0xC2, 0xC6, 0x66, 0x72, }, { 48,39,15, 48,38,15,} },
		{ { 0x8A, 0x7E, 0xC6, 0x66, 0x72, }, { 48,21,15, 48,38,15,} },
		{ { 0x6E, 0x92, 0x96, 0x66, 0x72, }, { 48,38,15, 48,38,15,} },
		{ { 0x6E, 0xC2, 0xC6, 0x66, 0x72, }, { 48,21,15, 48,38,15,} },
		{ { 0xB0, 0xB4, 0xB8, 0xBC, 0x52, }, { 39,21,15, 48,38,15,} },
	},

	0x10, // Arrival input

	ElecmanAscii, ElecmanChar
};
