static void FiremanAscii(Screen& s)
{
    GenericAsciiMaker(s,
// Sky
-1,'.', 0x00,0, // black background
// Background
-1,' ', 0x28,2, // red background
        0x29,2, // red background - shadow under a platform
-1,' ', 0x22,2, // window in red background
        0x23,2, // window in red background - shadow under a platform
        0x2B,2, // hole in a window
-1,'-', 0x27,2, // red thin horizontal pipe on black background
        0x26,2, // small detail in red thin pipe
        0x2F,2, // big detail in red thin pipe
-1,'=', 0x27,1, // green thin horizontal pipe on black background
        0x26,1, // small detail in green thin pipe
        0x2F,1, // big detail in green thin pipe
// Green background
-1,'b', 0x28,1, // green background
        0x29,1, // green background - shadow under a platform
// Lava (background)
-1,'~', 0x30,3, // lava surface
-1,'L', 0x38,3, // lava
-1,'z', 0xE4,0, // fence
// Lava (foreground)
-1,'E', 0x73,0,
        0x7B,0,         // horizontal lava tunnel 1x2, (stack these horizontally)
-1,'W', 0x6C,0, 0x6D,0, // vertical lava tunnel 2x1, (stack these vertically)
-1,'%', 0x74,0, 0x75,0,
        0x7C,0, 0x7D,0, // lava flow junction (2x2)
-1,'/', /*0,3,*/0x36,3,
        0x3F,3, 0x3E,3, // lava flow corner "/"
// Torch
-1,'>', 0x65,0, // right
-1,'^', 0x6E,0, // up
// Walls
-1,'#', 0x61,0, // wall
-1,'#', 0x60,0, // walkable surface
-1,'#', 0x77,0, // wall detail (best used in horizontal stretches)
-1,'#', 0x51,0, 0x52,0, 0x53,0, // Wily signature
// Red walls
-1,'&', 0x61,2, // wall
-1,'&', 0x60,2, // walkable surface
-1,'&', 0x77,2, // wall detail (best used in horizontal stretches)
// Wily door
-1,'D', 0x10,0,
// Ladders
-1,'|', 0x83,0,
-1,']', 0x82,0,

-2);
}

static void FiremanChar(Screen& s)
{
    unsigned randseed = 0;
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
            randseed += s.ascii[y*32+x];
    std::srand(randseed);
    
    char decorations[16*16] = { 0 };
    
    // Put windows somewhere
    for(unsigned y=0; y<16; ++y) 
        for(unsigned x=0; x<16; ++x)
            if(s.ascii[y*32+x]==' ' || s.ascii[y*32+x]=='b')
                if(rand()%30 == 0)
                    decorations[y*16+x] = 1;
    
    // Extend those windows randomly
    for(unsigned n=0; n<8; ++n)
    {
		for(unsigned y=1; y<15; ++y) 
			for(unsigned x=1; x<15; ++x)
			  if(rand()%3 == 0)
			    if(decorations[(y-1)*16+(x)]==1
			    || decorations[(y+1)*16+(x)]==1
			    || decorations[(y)*16+(x-1)]==1
			    || decorations[(y)*16+(x+1)]==1)
			        decorations[y*16+x] = 1;
    }
    

    // Put streaks in some walls
    for(unsigned n=0; n<5; ++n)
    {
        unsigned y = rand()%16;
        unsigned x1 = rand()%13;
        unsigned x2 = x1 + 2 + rand()%6;
        if(x2 > 15) x2 = 15;
        for(unsigned x=x1; x<=x2; ++x)
            if(decorations[y*16+x] == 0)
                decorations[y*16+x] = 2;
    }

    // Convert to tiles
    for(unsigned y=0; y<16; ++y) 
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c   = 0;
            unsigned char pal = 0;
            
            switch(s.ascii[y*32+x])
            {
                case '.':
                {
                    c   = 0x00; // black background
                    pal = 0x00;
                    break;
                }
                case '-':
                case '=':
                {
					pal = s.ascii[y*32+x]=='-' ? 0x02 : 0x01;
					c   = 0x27;
					switch(rand()%10)
					{
						case 0: c = 0x2F; break;
						case 1: c = 0x26; break;
					}                        
                    break;
                }
                case '>': c=0x65; pal=0; break;
                case '^': c=0x6E; pal=0; break;
                case ' ':
                case 'b':
                    if(y > 0 &&
                        (s.ascii[(y-1)*32+x] != '.'
                      && s.ascii[(y-1)*32+x] != ' '
                      && s.ascii[(y-1)*32+x] != '-'
                      && s.ascii[(y-1)*32+x] != 'b')
                      )
                    {
                        c = 0x29; // shadow
                        if(decorations[y*32+x] == 1)
                            c = 0x23;
                    }
                    else
                    {
                        c = 0x28;
                        if(decorations[y*32+x] == 1)
                        {
                            c = 0x22;
                            if(rand()%5 == 0) c = 0x2B;
                        }
                    }
                    pal = s.ascii[y*32+x]==' ' ? 0x02 : 0x01;
                    break;
                case 'E':
                {
                    unsigned ypos=0;
                    while(y > ypos && s.ascii[(y-ypos-1)*32+x]=='E') ++ypos;
                    c   = (ypos&1) ? 0x7B : 0x73;
                    pal = 0x00;
                    break;
                }
                case 'W':
                {
                    unsigned xpos=0;
                    while(x > xpos && s.ascii[y*32+(x-xpos-1)]=='W') ++xpos;
                    c   = (xpos&1) ? 0x6D : 0x6C;
                    pal = 0x00;
                    break;
                }
                case '%':
                {
                    unsigned xpos=0;
                    while(x > xpos && s.ascii[y*32+(x-xpos-1)]=='%') ++xpos;
                    unsigned ypos=0;
                    while(y > ypos && s.ascii[(y-ypos-1)*32+x]=='%') ++ypos;
                    c   = 0x74 + (xpos&1) + 8*(ypos&1);
                    pal = 0x00;
                    break;
                }
                case '#':
                case '&':
                    if(y > 0 &&
                        (s.ascii[(y-1)*32+x] != '#'
                      && s.ascii[(y-1)*32+x] != '&')
                      )
                        c = 0x60; // step
                    else
                        c = decorations[y*16+x] == 2 ? 0x77 : 0x61;
                    pal = s.ascii[y*32+x]=='#' ? 0 : 2;
                    break;
                case '|':
                case ']':
                    c   = 0x82 + rand()%2; // ladder
                    pal = 0x00;
                    break;
                case Ascii_WilyDoor:
                    c   = 0x10;
                    pal = 0x00;
                    break;
                case '~':
                    c = 0x30; pal = 0x03; break;
                case 'L':
                    c = 0x38; pal = 0x03; break;
                case 'z':
                    c = 0xE4; pal = 0x00; break;
                default:
                    printf("? '%c'\n", s.ascii[y*32+x]);
            }
            
            s.outmap[y*16+x].tile    = c;
            s.outmap[y*16+x].palette = pal;
        }
}

static Level Fireman =
{
	3, // level number
	"Fireman",

	// screens:
	{
		{
		"##    ....      "   "                "
		"##    ....      "   "                "
		"##    ....      "   "                "
		"##    ----      "   "                "
		"##|#############"   "       v       v"
		"##|           ##"   "                "
		"##|           ##"   "                "
		"##|           ##"   "                "
		"#############|##"   "       v        "
		"##           |##"   "                "
		"##           |##"   "                "
		"##           |##"   "                "
		"################"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"LLLLLLLLLLLLLLLL"   "                "
		"LLLLLLLLLLLLLLLL"   "                "
		, 0 // monster graphics index
		,LevelBegin| Right},
		{
		"..............##"   "     v          "
		"..............##"   "                "
		"      ........##"   "                "
		"      --------##"   "                "
		"#############|##"   "           v    "
		"      ....   |##"   "                "
		"        ..   |##"   "                "
		"        ..   |##"   "                "
		"|###############"   "      v         "
		"|               "   "                "
		"|               "   "                "
		"|               "   "                "
		"################"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"LLLLLLLLLLLLLLLL"   "                "
		"LLLLLLLLLLLLLLLL"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"####.........|##"   "                "
		"####.........|##"   "                "
		"####.........|##"   "                "
		"####.........|##"   "    E           "
		"######....#|####"   "                "
		"####...... |  ##"   "                "
		"##........ |  ##"   "                "
		"##........ |  ##"   "                "
		".......... |  ##"   "                "
		".......... |  ##"   "                "
		"......#^.. |  ##"   "                "
		"......##.. |  ##"   "                "
		"####..##..##..##"   "       I        "
		"~~~~~~##~~~~~~~~"   "                "
		"LLLLLL##LLLLLLLL"   "     d          "
		"LLLLLL##LLLLLLLL"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"##.|..........##"   "                "
		"##.|..........##"   "                "
		"##.|..........##"   "                "
		"##.|..........##"   "                "
		"##............##"   "                "
		"##....#######|##"   "                "
		"##...........|##"   "                "
		"##...........|##"   "  e             "
		"###^.........|##"   "                "
		"####......#|####"   "                "
		"####.......|..##"   "   I            "
		"####.......|..##"   "    e           "
		"#####^.....|..##"   "                "
		"#############|##"   "                "
		"#############|##"   "     I          "
		"#############|##"   "                "
		, 1 // monster graphics index
		,Up},
		{
		"##.............."   "                "
		"##.............."   "                "
		"##.............."   "                "
		"##.............."   "              PP"
		"##........######"   "                "
		"##........######"   "                "
		"##........######"   "                "
		"##........######"   "                "
		"##........######"   "                "
		"##.............."   "                "
		"##.|###^........"   "       I        "
		"##.|####..######"   "                "
		"##.|#####^######"   "       I        "
		"##.|############"   "                "
		"##.|############"   "         I      "
		"##.|############"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"........####...."   "                "
		"........###>...."   "            ] - "
		"#>....####......"   "  ] -           "
		"........##....#."   "                "
		"........##......"   "                "
		">....#..##......"   " ] -            "
		"........####..##"   "                "
		".###...###....##"   "                "
		"........##....##"   "                "
		"........##....##"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"####..#^..##..##"   "                "
		"####..##..##..##"   "                "
		"..###^##..##..##"   "       I        "
		"..##########..##"   "                "
		"..######......##"   "     I          "
		"..######......##"   "                "
		"########......##"   "        eee     "
		"############.|##"   "                "
		"############.|##"   "                "
		"############.|##"   "                "
		, 1 // monster graphics index
		,Down},
		{
		"############.|##"   "                "
		"############.|##"   "                "
		"####..######.|##"   "                "
		"####..######.|##"   "                "
		"####.........|##"   "                "
		"####.........|##"   "                "
		"############.|##"   "                "
		"####.........|##"   "                "
		"####..........##"   "                "
		"####..........##"   "                "
		"##............##"   "                "
		"##............##"   "                "
		"##......#^..#^##"   "                "
		"##.|##..########"   "                "
		"##.|##..########"   "      d  I   I  "
		"##.|############"   "                "
		, 1 // monster graphics index
		,Down},
		{
		"##.|############"   "                "
		"##.|############"   "                "
		"##.|..##########"   "                "
		"##.|..##########"   "                "
		"##.............."   "                "
		"##.............."   "                "
		"##............zz"   "                "
		"##...........|##"   "                "
		"##...........|.."   "                "
		"##...........|.."   "                "
		"##......#^...|.."   "                "
		"##......##..##.."   "                "
		"##..#^..##......"   "         I      "
		"##~~##~~##~~~~~~"   "                "
		"##LL##LL##LLLLLL"   "     I     d    "
		"##LL##LL##LLLLLL"   "                "
		, 1 // monster graphics index
		,RestartPointA| Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"zzzz......zzzz.."   "                "
		"####.....|####.."   "                "
		".........|......"   "                "
		".........|......"   "                "
		"........##......"   "                "
		"................"   "                "
		"..............#^"   "                "
		"~~~~~~~~~~~~~~##"   "                "
		"LLLLLLLLLLLLLL##"   "     d         I"
		"LLLLLLLLLLLLLL##"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"###########|WW##"   "                "
		"###########|WW##"   "                "
		"########...|WW##"   "                "
		"########...|WW##"   "                "
		"...........|WW##"   "                "
		"...........|WW##"   "                "
		"...........|WW##"   "                "
		"...........|WW##"   "                "
		"...........|WW##"   "                "
		"...........|WW##"   "                "
		"..#^..######WW##"   "                "
		"..##........WW##"   "                "
		"..##............"   "   I            "
		"~~##~~~~~~~~~~~~"   "                "
		"LL##LLLLLLLLLLLL"   "    d           "
		"LL##LLLLLLLLLLLL"   "                "
		, 1 // monster graphics index
		,Up},
		{
		"##  |     ..####"   "                "
		"##  |     ..  ##"   "                "
		"##  |     ..  ##"   "                "
		"##  |     ..  ##"   "                "
		"##  |     ..  ##"   "                "
		"##  %%EEEE%%| ##"   "                "
		"##  %%EEEE%%| ##"   "                "
		"##  ..      | ##"   "                "
		"##  ..      | ##"   "                "
		"###|%%EEEEEE%%##"   "                "
		"###|%%EEEEEE%%##"   "                "
		"## |        WW##"   "                "
		"## |        WW##"   "                "
		"###########|WW##"   "                "
		"###########|WW##"   "                "
		"###########|WW##"   "                "
		, 1 // monster graphics index
		,Up},
		{
		"#..............."   "                "
		"#..............."   "                "
		"#.......        "   "                "
		"#-------        "   "                "
		"#.......      ##"   "                "
		"#.......      ##"   "                "
		"#.....      ####"   "                "
		"#.....      ####"   "                "
		"#.....    ######"   "                "
		"#-----    ####  "   "                "
		"#.....  ######  "   "              s "
		"#-----  ######  "   "                "
		"#...  ##########"   "                "
		"##..|#####?/~~~~"   "                "
		"##..|#####//LLLL"   "                "
		"##..|#####LLLLLL"   "                "
		, 2 // monster graphics index
		,Right},
		{
		"..............##"   "                "
		"..............##"   "                "
		"      ........##"   "                "
		"      --------##"   "             f  "
		"#############|##"   "                "
		"      ....   |##"   "                "
		"        ..   |##"   "                "
		"        ..   |##"   " f              "
		"|###############"   "                "
		"|               "   "                "
		"|               "   "                "
		"|               "   "               f"
		"################"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"LLLLLLLLLLLLLLLL"   "                "
		"LLLLLLLLLLLLLLLL"   "                "
		, 2 // monster graphics index
		,Right},
		{
		"..        ......"   "                "
		"..        ......"   "                "
		"          ......"   "                "
		"          ......"   "  P             "
		"  |         ...."   "                "
		"  |         ...."   "                "
		"  |             "   "                "
		"  |             "   "                "
		"  |             "   "                "
		"  |             "   "                "
		"  |             "   "                "
		"  |             "   "                "
		"####  ##  ##  ##"   "                "
		"~~~~~~~~~~~~~~##"   "                "
		"LLLLLLLLLLLLLL##"   "        d   d   "
		"LLLLLLLLLLLLLL##"   "        s       "
		, 2 // monster graphics index
		,Right},
		{
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		"..    .......###"   "                "
		"..    .......###"   "                "
		"        ......DD"   "              SS"
		"        ......DD"   "              SS"
		"          ....DD"   "              SS"
		"          ....DD"   "              SS"
		"          #^#^##"   "                "
		"          ######"   "                "
		"################"   "           I I  "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 2 // monster graphics index
		,Shutter},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"bbbbbbbbbbbbbbbb"   "           .    "
		"bbbbbbbbbbbbbbbb"   "                "
		"bbbbbbbbbbbbbbbb"   "                "
		"bbbbbbbbbbbbbbbb"   "                "
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		, 0 // monster graphics index
		,RestartPointB| Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"bbbbbbbbbbbbbbbb"   "     .        . "
		"bbbbbbbbbbbbbbbb"   "                "
		"bbbbbbbbbbbbbbbb"   "                "
		"bbbbbbbbbbbbbbbb"   "                "
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"bbbbbbbbbbbbbbbD"   "      .        S"
		"bbbbbbbbbbbbbbbD"   "               S"
		"bbbbbbbbbbbbbbbD"   "               S"
		"bbbbbbbbbbbbbbbD"   "               S"
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		, 0 // monster graphics index
		,Shutter},
		{
		"################"   "                "
		"################"   "                "
		"#bbbbbbbbbbbbbb#"   "                "
		"#bbbbbbbbbbbbbb#"   "                "
		"#bbbbbbbbbbbbbb#"   "                "
		"#bbbbbbbbbbbbbb#"   "                "
		"bbbbbbbbbbbbbbb#"   "                "
		"bbbbbbbbbbbbbbb#"   "                "
		"bbbbbbbbbbbbbbb#"   "                "
		"bbbbbbbbbbbbbbb#"   "                "
		"#bbbbbbbbbbbbbb#"   "                "
		"#bbbbbbbbbbbbbb#"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 3 // monster graphics index
		,End},
	},

	// palette
	{
		15, 48, 16,  6, 
		15, 27, 11,  0, 
		15, 39, 22,  6, 
		15, 38, 22,  6, 

		15, 44, 17, 15, 
		15, 32, 56, 15, 
		15, 39, 21, 15, 
		15, 48, 17, 15, 

		15, 48, 16,  6, 
		15, 32, 16,  0, 
		15, 39, 22,  6, 
		15, 48, 16, 22, 
	},

	// after doors palette
	{
		15, 32, 39, 23, 
		15, 32, 39, 33, 
		15, 15, 15, 12, 
		15, 40, 41, 24, 

	},

	// actives
	{
	},

	// enemies
	{
		Enemy( 0, 0x80,0xE0, 0x32 ),
		Enemy( 10, 0xD0,0x04, 0xE ),
		Enemy( 11, 0xB0,0xFF, 0xE ),
	},

	// level gfx (id, count) (max 16); id is block*4+bank
	{
		{ 0x40, 8 }, { 0x7A, 2 }, { 0xDE, 1 }, { 0x7A, 2 }, 
		{ 0xE2, 1 }, { 0xD8, 1 }, { 0x5E, 1 }, { 0xC0, 8 }, 
		{ 0xA1, 8 }, 
	},

	// monster gfx tables (max 11) (5 tiles, 2*3bytepalette)
	{
		{ { 0x7A, 0xFE, 0xDE, 0x7A, 0x7E, }, { 39,21,15, 48,17,15,} },
		{ { 0x7A, 0xFE, 0xDE, 0xC6, 0x42, }, { 39,21,15, 48,17,15,} },
		{ { 0x7A, 0xFE, 0xDE, 0x62, 0x72, }, { 39,21,15, 48,17,15,} },
		{ { 0x90, 0x94, 0x98, 0x5A, 0x42, }, { 32,21,15, 39,21,15,} },
	},

	0x00, // Arrival input

	FiremanAscii, FiremanChar
};
