/*

ti pa

30.1 = 1st row cloud

32.1
01.1 = 2-3 row background

31.1 {30.1} 31.1
33.1 {34.1} 35.1 = 2-3 row cloud

01.1 = other row background

33.1 {34.1} 35.1 = other row cloud

3F.2 = frozen grass

3E.2 = frozen tree branch

38.2 39.2 3A.2 
3B.2 3C.2 3D.2 = frozen tree leaves

A0.0 A1.0 = pillar surface
68.0 69.0 = pillar

         66.0 67.0
         6E.0 6F.0
77.0 ... 76.0 6F.0 = metal object

A2.0 = vertical metal bar surface
6A.0 = vertical metal bar
6B.0 = vertical metal bar detail

E3.3 = water surface
C1.3 = water

2D.3 = inside
2C.3 = inside shadow

24.0 25.0 = plastic pit (background)

*/

static void IcemanAscii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            unsigned char p = s.outmap[y*16+x].palette;
            
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                // sky:
                case 0x32:
                case 0x01:
                case 0x31: case 0x30:
                case 0x33: case 0x34: case 0x35: // clouds, sky.
                case 0x00: //void
                case 0x3E: case 0x3F:
                case 0x38 ... 0x3A:
                case 0x3B ... 0x3D: // trees and grass
                    ch = '.';
                    break;
                
                case 0xA0: case 0xA1:
                case 0x68: case 0x69: // pillar
                case 0xA2: case 0x6A: case 0x6B: // metal part in pillar
                    ch = '#'; // pillar
                    break;
                case 0x51: case 0x52: case 0x53: // wily signature
                    ch = '#';
                    break;
                case 0x77: // horizontal metal bar (part of "metal object")
                    ch = '=';
                    break;
                case 0x66: case 0x67:
                case 0x6E: case 0x6F:
                case 0x76: // metal object
                    ch = 'M';
                    break;
                case 0x2C: case 0x2D:
                    ch = ' '; // inside
                    break;
                case 0xC1: case 0xE3:
                    ch = '~'; // water
                    break;
                case 0x24: case 0x25:
                    ch = 'V';
                    break;
                    
                case 0x10:
                    ch = Ascii_WilyDoor;
                    break;
                
                // There are no ladders, gutsblock or spikes in Iceman level.
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);      
        }
}

#include "treegen.hh"

namespace IceUtil
{
    bool isFree(unsigned Wid, unsigned Hei, unsigned BranchWid,
                unsigned x, unsigned y, unsigned BranchHei,
                const unsigned char* screen,
                const char* ground)
    {
        unsigned mask = 0;
		for(unsigned i = 0; i < Wid; ++i)
			for(unsigned j = 0; j < Hei; ++j)
			{
				if(ground[x+i + 32*(y+j)] != '.') return false;
				if(screen[x+i + 16*(y+j)] != 0) ++mask;
	        }
	    if(mask != 0 && mask != Wid*Hei) return false;

		const unsigned bx = x + Wid/2 - BranchWid/2;
		const unsigned by = y + Hei;

		for(unsigned iy=0; iy<BranchHei; ++iy)
			for(unsigned ix = 0; ix < BranchWid; ++ix)
				if(ground[bx+ix + 32*(by+iy)] != '.') return false;
		return true;
    }

    void CreateTrees(unsigned char dest[16*16], const char ground[32*16])
	{
		static const unsigned char SmallBall[2][3] =
		{
			{ 0x38, 0x39, 0x3A },
			{ 0x3B, 0x3C, 0x3D }
		};
		static const unsigned char SmallBranch[1] = { 0x3E };
		
		for(unsigned y=10; y<16; ++y)
		    for(unsigned tryn=0; tryn<40; ++tryn)
		    {
		        unsigned x = rand()%14 + 1;
		        char h = ground[y*32+x];
		        bool here = (h=='#' || h=='M' || h=='=');
		        bool above= ground[(y-1)*32+x] == '.';
		        if(here && above)
		        {
		            unsigned hei = std::rand()%2 + 1;
		            
		            unsigned beginY = y-2-hei;
		            if(isFree(3,2,1, x,beginY,hei, dest,ground))
		                CreateTree<3,2,1> (SmallBall,SmallBranch, dest, x,beginY, hei);
		            else
		                dest[(y-1)*16+x] = 0x3F;
		        }
		    }
	}
}


static void IcemanChar(Screen& s)
{
    unsigned randseed = 0;
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
            randseed += s.ascii[y*32+x];
    std::srand(randseed);
    
    unsigned char decorations[16*16] = {0};
    
    for(unsigned y=2; y<6; ++y)
    {
        for(unsigned x=0; x<16; )
        {
            unsigned nextcloud = 0 + rand()%(y*2);
            x += nextcloud;
            if(x >= 14) break;
            unsigned width = 2 + rand()%(1+rand()%3);
            if(x+width > 16) width=16-x;
            for(unsigned xm=0; xm<width; ++xm)
            {
                char c = 2;
                if(xm==0) c=1; else if(xm==width-1) c=3;
                decorations[y*16+x+xm] = c;
            }
            x += width;
        }
    }
    
    IceUtil::CreateTrees(decorations, s.ascii);

    for(unsigned y=1; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
            if(s.ascii[y*32+x] == '#'
            && s.ascii[(y-1)*32+x] != '#'
            && rand()%6 == 0)
            {
                for(unsigned my=0; y+my<16 && s.ascii[(y+my)*32+x]=='#'; ++my)
                {
                    decorations[(y+my)*16+x] = 1;
                    if(my>0 && rand()%4 == 0)break;
                }
            }
    
    
    // Convert to tiles
    for(unsigned y=0; y<16; ++y) 
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c   = 0;
            unsigned char pal = 0;
            
            switch(s.ascii[y*32+x])
            {
                case '.':
                {
                    // background
                    pal = 1;
                    
                    switch(y)
                    {
                        case 0:
                            c = 0x30; break;
                        case 1:
                            c = 0x32; if(decorations[2*16+x]) c = decorations[2*16+x]==2 ? 0x30 : 0x31; break;
                        case 2:
                            c = 0x01; if(decorations[2*16+x]) c = 0x33+decorations[2*16+x]-1; break;
                        default:
                        {
                            char deco = decorations[y*16+x];
                            c = 0x01;
                            if(deco>=1 && deco<=3) c = 0x33+decorations[y*16+x]-1;
                            else if(deco>3) { c=deco; pal=2; }
                            break;
                        }
                    }
                    break;
                }
                case '#':
                {
                    pal = 0;
                    bool decor = decorations[y*16+x] == 1;
                    bool up = y>0 && s.ascii[(y-1)*32+x]!='#';
                    if(up) c = decor ? 0xA2 : (0xA0 + (x&1));
                    else c = decor ? (0x6A + !(rand()%4))
                                   : (0x68 + (x&1));
                    break;
                }
                case '=':
                    c = 0x77;
                    pal = 0;
                    break;
                case 'M':
                {
                    pal = 0;
                    bool left  = x>0  && s.ascii[y*32+(x-1)]=='M';
                    bool up    = y>0  && s.ascii[(y-1)*32+x]=='M';
                    bool down  = y<15 && s.ascii[(y+1)*32+x]=='M';
                    if(!up)
                        c = 0x66 + left;
                    else if(down)
                        c = 0x6E + left;
                    else
                        c = left ? 0x6F : 0x76;
                    break;
                }
                case 'V':
                    pal = 0;
                    c = 0x24 + (x&1);
                    break;
                case '~':
                {
                    bool up = y>0 && s.ascii[(y-1)*32+x]=='~';
                    pal = 3;
                    c = up ? 0xC1 : 0xE3;
                    break;
                }
                case ' ':
                {
                    bool up  = y==0 || s.ascii[(y-1)*32+x]==' ';
                    c   = up ? 0x2D : 0x2C;
                    pal = 3;
                    // inside
                    break;
                }
                case Ascii_Ladder:
                    c   = 0x82; // ladder
                    pal = 0x00;
                    break;
                case Ascii_Spike:
                    c   = 0xC4; // spike
                    pal = 0x01;
                    break;
                case Ascii_WilyDoor:
                    c   = 0x10;
                    pal = 0x00;
                    break;
                default:
                    printf("? '%c'\n", s.ascii[y*32+x]);
            }
            
            s.outmap[y*16+x].tile    = c;
            s.outmap[y*16+x].palette = pal;
        }
}

static Level Iceman =
{
	1, // level number
	"Iceman",

	// screens:
	{
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,LevelBegin| Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "R               "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"............####"   "                "
		"............####"   "           R    "
		"....############"   "                "
		"....############"   "   R            "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"....######......"   "                "
		"....######......"   "   R            "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "               i"
		"~~~~MM~~~~MM~~~~"   "                "
		"~~~~MM~~~~MM~~~~"   "         f      "
		"====MM====MM===="   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~MM~~~~MM~~~~"   "                "
		"~~~~MM~~~~MM~~~~"   "         f      "
		"====MM====MM===="   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~~~~~~~~~~~~~"   "                "
		"~~~~MM~~~~MM~~~~"   "                "
		"~~~~MM~~~~MM~~~~"   "         f      "
		"====MM====MM===="   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"~~~~~~~~~~~~####"   "                "
		"~~~~~~~~~~~~####"   "                "
		"~~~~~~~~########"   "                "
		"~~~~~~~~########"   "       f        "
		"~~~~############"   "                "
		"~~~~############"   "                "
		"====############"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"....######......"   "                "
		"....######......"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "        i       "
		, 0 // monster graphics index
		,Right},
		{
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"######  ####VV##"   "                "
		"######  ####VV##"   "                "
		"######  ####VV##"   "                "
		"####      ##VV##"   "                "
		"####      ##VV##"   "    c           "
		"            VV##"   "                "
		"            VV##"   "E c             "
		"############VV##"   "                "
		"############VV##"   "                "
		"############VV##"   "                "
		, 0 // monster graphics index
		,Down},
		{
		"##          VV##"   "                "
		"##          VV##"   "                "
		"##          VV##"   "                "
		"##VV##      VV##"   "                "
		"##VV##        ##"   "                "
		"##VV##        ##"   "                "
		"##VV##        ##"   "                "
		"##VV##        ##"   "                "
		"##VV##        ##"   "                "
		"##VV##        ##"   "                "
		"##VV##        ##"   "                "
		"##VV##        ##"   "                "
		"##VV##~~~~~~~~##"   "                "
		"##VV##~~~~~~~~##"   "           f    "
		"##VV############"   "                "
		"##VV############"   "                "
		, 1 // monster graphics index
		,Down| RestartPointA},
		{
		"##VV##        .."   "                "
		"##VV##        .."   "                "
		"##VV          .."   "                "
		"##VV          .."   "                "
		"##            .."   "                "
		"##            .."   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"####~~~~~~~~~~##"   "                "
		"####~~~~~~~~~~##"   "         f      "
		"################"   "                "
		"################"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"....######......"   "                "
		"....######......"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "    L           "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "          L    L"
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "         P    L "
		"........##......"   "                "
		"........##......"   "      L         "
		"........##......"   "                "
		"........##......"   "                "
		"........##......"   "                "
		"........##......"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "       i        "
		"................"   "          L     "
		"................"   "               L"
		"................"   "    L           "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"############VV##"   "                "
		"############VV##"   "                "
		"############VV##"   "                "
		"############VV##"   "                "
		"############VV##"   "                "
		"####        VV##"   "                "
		"####        VV##"   "    U           "
		"############VV##"   "                "
		"############VV##"   "                "
		"############VV##"   "i               "
		, 1 // monster graphics index
		,Down},
		{
		"############VV##"   "                "
		"############VV##"   "                "
		"############VV##"   "                "
		"############VV##"   "                "
		"##          VV##"   "                "
		"##          VV##"   "                "
		"##          VV##"   "                "
		"##          VV##"   "   pp pe ee     "
		"############VV##"   "                "
		"############VV##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##VV############"   "                "
		"##VV############"   "                "
		"##VV############"   "                "
		"##VV############"   "                "
		, 2 // monster graphics index
		,Down},
		{
		"##VV            "   "                "
		"##VV            "   "                "
		"##VV            "   "                "
		"##VV            "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "              t "
		"##              "   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 2 // monster graphics index
		,Right},
		{
		"             ###"   "                "
		"             ###"   "                "
		"             ###"   "                "
		"             ###"   "                "
		"             ###"   "                "
		"             ###"   "                "
		"              DD"   "              SS"
		"              DD"   "              SS"
		"              DD"   "              SS"
		"              DD"   "              SS"
		"          ######"   "                "
		"          ######"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 2 // monster graphics index
		,Shutter},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"################"   "               i"
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		, 2 // monster graphics index
		,RestartPointB| Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		, 2 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"               D"   "               S"
		"               D"   "               S"
		"               D"   "               S"
		"               D"   "               S"
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "              i "
		, 2 // monster graphics index
		,Shutter},
		{
		"################"   "                "
		"################"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"               #"   "                "
		"               #"   "                "
		"               #"   "                "
		"               #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 3 // monster graphics index
		,End},
	},

	// palette
	{
		15, 32, 33, 17, 
		15, 32, 60, 44, 
		15, 32, 33, 44, 
		15, 48,  9, 28, 

		15, 44, 17, 15, 
		15, 32, 56, 15, 
		15, 48, 42, 15, 
		15, 48, 38, 15, 

		15, 32, 33, 17, 
		15, 32, 60, 44, 
		15, 32, 33, 44, 
		15, 48, 16, 32, 
	},

	// after doors palette
	{
		15, 32, 16, 17, 
		15, 32, 16, 33, 
		15, 41, 25,  7, 
		15, 39,  6, 33, 

	},

	// actives
	{
		Active( 0, 10, 0x60,0x90, 0x70,0xA0 ),
		Active( 0, 10, 0x80,0x30, 0x90,0x40 ),
		Active( 0, 10, 0x80,0x80, 0x90,0x90 ),
		Active( 0, 10, 0x80,0xB0, 0x90,0xC0 ),
		Active( 0, 10, 0xA0,0x60, 0xB0,0x70 ),
		Active( 0, 10, 0xB0,0xA0, 0xC0,0xB0 ),
		Active( 0, 10, 0xD0,0x80, 0xE0,0x90 ),
		Active( 0, 11, 0x20,0x50, 0x30,0x60 ),
		Active( 0, 11, 0x30,0x90, 0x40,0xA0 ),
		Active( 0, 11, 0x50,0x70, 0x60,0x80 ),
		Active( 0, 11, 0x60,0xA0, 0x70,0xB0 ),
		Active( 0, 11, 0x70,0x30, 0x80,0x40 ),
		Active( 0, 11, 0x70,0x60, 0x80,0x70 ),
		Active( 0, 11, 0x90,0x70, 0xA0,0x80 ),
		Active( 0, 11, 0x90,0xB0, 0xA0,0xC0 ),
		Active( 0, 11, 0xA0,0x90, 0xB0,0xA0 ),
		Active( 0, 11, 0xB0,0x30, 0xC0,0x40 ),
		Active( 0, 11, 0xC0,0xB0, 0xD0,0xC0 ),
	},

	// enemies
	{
		Enemy( 4, 0x80,0x02, 0xFF ),
		Enemy( 4, 0x82,0xFE, 0x1F ),
		Enemy( 8, 0x00,0x00, 0xFF ),
		Enemy( 10, 0xE0,0x80, 0x10 ),
		Enemy( 11, 0xE0,0x80, 0x10 ),
	},

	// level gfx (id, count) (max 16); id is block*4+bank
	{
		{ 0x40, 8 }, { 0xA2, 3 }, { 0x76, 1 }, { 0x72, 1 }, 
		{ 0xE2, 1 }, { 0xD8, 1 }, { 0x5E, 1 }, { 0xC0, 8 }, 
		{ 0x61, 8 }, 
	},

	// monster gfx tables (max 11) (5 tiles, 2*3bytepalette)
	{
		{ { 0xA2, 0xA6, 0xAA, 0x76, 0x72, }, { 48,42,15, 48,38,15,} },
		{ { 0xD6, 0xB6, 0xBA, 0x76, 0x72, }, { 44,17,15, 48,38,15,} },
		{ { 0x6E, 0xC2, 0xC6, 0x76, 0x72, }, { 48,21,15, 48,38,15,} },
		{ { 0xA0, 0xA4, 0xA8, 0xAC, 0x46, }, { 32,33,15, 48,38,15,} },
	},

	0x00, // Arrival input

	IcemanAscii, IcemanChar
};
