static void Wily1Ascii(Screen& s)
{
    GenericAsciiMaker(s,
// Boss room
-1,'X', 0x00,1,
// Suspense
-1,'V', 0x00,0, // entire black background
// Sky
-1,'.', 0x01,1,
// Decorations
-1,' ', 0x16,2, 0x17,2, // bg, shadow
-1,' ', 0x27,1, // grass
// Walls
-1,'#', 0x48,3, 0x49,3, 0x4A,3,
        0x50,3, 0x51,3, 0x52,3, // block type 1 (middle piece repeatable)
-1,'R', 0x4B,0, 0x4C,0, 0x4D,0,
        0x53,0, 0x54,0, 0x55,0, // block type 2 (middle piece repeatable)
-1,'#', 0x66,0, // floor outdoors
-1,'#', 0x4E,3, // floor indoors
// Spikes up, down (black bg)
-1,'^', 0xAE,0, 
-1,'^', 0xBF,0,
// Ladder
-1,'|', 0xC5,0,
// Gutsblock
-1,'G', 0x38,0, 0x39,0,
        0x3A,0, 0x3B,0,
-2);
}

#include "legofill.hh"

static bool Wily1Air(char c)
{
    return c==' ' || c=='V';
}
static bool Wily1Wall(char c)
{
    return c=='#' || c=='R';
}

static void Wily1Char(Screen& s)
{
    std::srand(0);
    
    unsigned char decorations[16*16] = {0};
    
    unsigned char Lego1[2][3] = {{0x48,0x49,0x4A},{0x50,0x51,0x52}};
    unsigned char Lego2[2][3] = {{0x4B,0x4C,0x4D},{0x53,0x54,0x55}};
    
    LegoFill(decorations, s.ascii,'#', Lego1,Lego1);
    LegoFill(decorations, s.ascii,'R', Lego2,Lego2);

    // Find single-block-height platforms.
    for(unsigned y=1; y<15; ++y) 
        for(unsigned x=0; x<16; ++x)
        {
            if(s.ascii[(y)*32+x]=='#'
            && Wily1Air(s.ascii[(y-1)*32+x])
            && Wily1Air(s.ascii[(y+1)*32+x]))
            {
                // platform that is wholly in air!
                
                // Find the left edge
                unsigned begin = x;
                while(begin > 0)
                {
                    unsigned testx = begin-1;
                    bool air = Wily1Air(s.ascii[(y-1)*32+testx]);
                    bool wall=!Wily1Air(s.ascii[(y  )*32+testx]);
                    if(!wall || !air) break;
                    begin=testx;
                }
                // Find the right edge
                unsigned end = x;
                while(end < 15)
                {
                    unsigned testx = end+1;
                    bool air = Wily1Air(s.ascii[(y-1)*32+testx]);
                    bool wall=!Wily1Air(s.ascii[(y  )*32+testx]);
                    if(!wall || !air) break;
                    end=testx;
                }
                for(unsigned xpos=begin; xpos<=end; ++xpos)
                    decorations[y*16+xpos] = 0x4E; // platform
		        x = end;
            }
        }

    // Convert to tiles
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = 0;
            unsigned char p = 0;
            
            /* Wily1 palettes:
             *   
             * 0 = solid
             * 1 = blacks
             * 2 = blues
             * 3 = blue,gray
             */
            
            switch(s.ascii[y*32+x])
            {
                case 'X': // boss room bg
                    c   = 0x00;
                    p   = 1;
                    break;
                case 'V': // black bg
                    c   = 0x00;
                    p   = 0;
                    break;
                case '.': // sky
                    c   = 0x01;
                    p   = 1;
                    break;
                case ' ':
                {
                    // If above is sky, this is grass.
                    if(y>0 && s.ascii[(y-1)*32 + x]=='.')
                    {
                        c = 0x27;
                        p = 1;
                        break;
                    }
                    c = 0x17;
                    p = 2;
                    // If above is solid, this is shadow.
                    if(y>0 && Wily1Wall(s.ascii[(y-1)*32 + x]))
                    {
                        c = 0x16;
                    }
                    break;
                }
                case Ascii_Spike:
                {
                    // If below is solid, it's up. Otherwise it's down.
                    if(y<16 && Wily1Wall(s.ascii[(y+1)*32 + x]))
                        c = 0xAE;
                    else
                        c = 0xBF;
                    p = 0;                    
                    break;
                }
                case Ascii_Ladder:
                {
                    c = 0xC5;
                    p = 0;
                    break;
                }
                case Ascii_Guts:
                {
                    c = 0x38 + (x&1) + (y&1)*2; // gutsblock
                    p = 0;
                    break;
                }
                case '#':
                case 'R':
                {
                    char deco = decorations[y*16+x];
                    p = 3;
                    c = 0x66; if(deco) c = deco;
                    if((c >= 0x4B && c <= 0x4D) || (c >= 0x53 && c <= 0x55)) p = 0;
                    break;
                }
                default:
                    printf("? '%c'\n", s.ascii[y*32+x]);
            }
            
            s.outmap[y*16+x].tile    = c;
            s.outmap[y*16+x].palette = p;
        }
}

static Level Wily1 =
{
	6, // level number
	"Wily1",

	// screens:
	{
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"                "   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,LevelBegin| Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "t               "
		"                "   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "               t"
		"                "   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"......####......"   "                "
		"......####......"   "                "
		"....########...."   "                "
		"    ########    "   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "       t        "
		"................"   "                "
		"......####......"   "                "
		"......####......"   "                "
		"....########...."   "                "
		"    ########    "   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"..........##RRRR"   "                "
		"..........##RRRR"   "                "
		"..........####RR"   "                "
		"..........####RR"   "                "
		"..........######"   "                "
		"..........######"   "                "
		"..........##RRRR"   "                "
		"..........##RRRR"   "                "
		"..........GGGGGG"   "                "
		"..........GGGGGG"   "                "
		"..........GG##GG"   "                "
		"          GGRRGG"   "                "
		"############RR##"   "                "
		"##########RRRRRR"   "                "
		"################"   "                "
		"################"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"####RRRR#####|##"   "                "
		"####RRRR#####|##"   "                "
		"RR######### V|##"   "                "
		"RR######### V|##"   "                "
		"#########V  V#RR"   "                "
		"#########V  VRRR"   "                "
		"RR####V  V##X###"   "                "
		"RR####V  VRRX###"   "                "
		"      V##V######"   "                "
		"      VRRV######"   "                "
		"    ##X###RRRR##"   "            I   "
		"    RRX###RRRR##"   "                "
		"########RRRR####"   "         I      "
		"RRRR####RRRR####"   "                "
		"##########RRRR##"   "      I         "
		"##########RRRR##"   "                "
		, 0 // monster graphics index
		,Up},
		{
		"##|###RRRR##RR##"   "                "
		"##|###RRRR##RR##"   "                "
		"##|#####   #####"   "                "
		"##|#####   #####"   "         E      "
		"##|   ####    ##"   "                "
		"##|   ####    ##"   "                "
		"##|        ##|##"   "                "
		"##|        ##|##"   "                "
		"##           |##"   "                "
		"##   ##      |##"   "                "
		"##   ##      |##"   "                "
		"##   ##  #  #|##"   "                "
		"##^^^##^^#^^#|##"   "                "
		"#############|##"   "                "
		"######RR#####|##"   "                "
		"######RR#####|##"   "                "
		, 1 // monster graphics index
		,Up},
		{
		"RRRR####RR######"   "                "
		"RRRR####RR######"   "                "
		"######RR####RR##"   "                "
		"######RR####RR##"   "                "
		"RR##RRRRRR####RR"   "                "
		"RR##RRRRRR####RR"   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##            ##"   "       j   j   j"
		"##            RR"   "                "
		"##|#############"   "                "
		"##|RRRRRRRRRRR##"   "                "
		"##|###RR##RRRRRR"   "                "
		"##|###RR##RRRRRR"   "                "
		, 2 // monster graphics index
		,Right},
		{
		"RR########RRRR##"   "                "
		"RR########RRRR##"   "                "
		"        GGGG  ##"   "                "
		"        GGGG  ##"   "E  j   j        "
		"############  ##"   "                "
		"RRRRRRRRRRRR  ##"   "                "
		"              ##"   "                "
		"              ##"   "             j  "
		"    ############"   "                "
		"    RRRRRRRRRR##"   "                "
		"  GGGGGG      ##"   "                "
		"  GGGGGG      ##"   "                "
		"#############|##"   "                "
		"RRRRRRRRRRRRR|##"   "                "
		"########RR###|##"   "                "
		"########RR###|##"   "                "
		, 2 // monster graphics index
		,Down},
		{
		"##            ##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##  ############"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"####  #  #  # ##"   "               s"
		"####^^#^^#^^# ##"   "                "
		"############# ##"   "                "
		"##            ##"   "                "
		"##            ##"   "                "
		"##|#############"   "                "
		"##|#######RRRRRR"   "                "
		"##|#######RRRRRR"   "                "
		, 2 // monster graphics index
		,Down| RestartPointA},
		{
		"## #########RRRR"   "                "
		"## #########RRRR"   "                "
		"##  ^^^^^^^^^^^^"   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "              T "
		"##              "   "                "
		"####            "   "                "
		"##RR^^^^^^^^^^^^"   "                "
		"########RR######"   "                "
		"########RR######"   "                "
		, 3 // monster graphics index
		,Right},
		{
		"####RR### ###|##"   "                "
		"####RR### ###|##"   "                "
		"^^^^^^^^^    |##"   "                "
		"             |##"   "                "
		"             |##"   "                "
		"             |##"   "                "
		"             |##"   "                "
		"             |##"   "                "
		"             |##"   "                "
		"             |##"   "                "
		"             |##"   "   L    L       "
		"             |##"   "           pp   "
		"          ######"   "                "
		"^^^^^^^^^^RRRR##"   "                "
		"########RR######"   "                "
		"########RR######"   "                "
		, 3 // monster graphics index
		,Up},
		{
		"##|#############"   "                "
		"##|#############"   "                "
		"##|   ##      ##"   "                "
		"##|   ##      ##"   "                "
		"##    ##  ##  ##"   "                "
		"##    ##  RR  ##"   "                "
		"##    ##  ##  ##"   "                "
		"##    ##  ##  ##"   "                "
		"##    ##  ##  ##"   "                "
		"##    ##  ##  ##"   "                "
		"##        ##  ##"   "                "
		"##        ##  ##"   "                "
		"##        ##  ##"   "                "
		"#########V###|##"   "                "
		"#########V###|##"   "                "
		"#########V###|##"   "                "
		, 1 // monster graphics index
		,Up},
		{
		"RRRR####RR######"   "                "
		"RRRR####RR######"   "                "
		"######RR####RR##"   "                "
		"######RR####RR##"   "                "
		"RR##RRRRRR####RR"   "                "
		"RR##RRRRRR####RR"   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##              "   "                "
		"##            ##"   "                "
		"##            RR"   "                "
		"##|#############"   "                "
		"##|RRRRRRRRRRR##"   "                "
		"##|###RR##RRRRRR"   "                "
		"##|###RR##RRRRRR"   "                "
		, 1 // monster graphics index
		,Shutter| RestartPointB},
		{
		"RR##########RR##"   "                "
		"RR##########RR##"   "                "
		"XXXXXXXXXXXXXXX#"   "                "
		"XXXXXXXXXXXXXXX#"   "                "
		"XXXXXXXXXXXXXXX#"   "                "
		"XXXXXXXXXXXXXXX#"   "                "
		"XXXXXXXXXXXXXXX#"   "                "
		"XXXXXXXXXXXXXXX#"   "                "
		"XXXXXXXXXXXXXXX#"   "                "
		"XXXXXXXXXXXXXXX#"   "                "
		"################"   "                "
		"RRRRRRRRRRRRRRR#"   "                "
		"########RR##RRRR"   "                "
		"########RR##RRRR"   "                "
		"######RRRRRR####"   "                "
		"######RRRRRR####"   "                "
		, 4 // monster graphics index
		,End},
	},

	// palette
	{
		15, 32,  0,  2, 
		15, 43, 25, 44, 
		15, 23,  7, 15, 
		15, 32,  0, 38, 

		15, 44, 17, 15, 
		15, 32, 56, 15, 
		15, 39, 21, 15, 
		15, 48, 17, 15, 

		15, 32,  0,  2, 
		15,  0, 16, 32, 
		15, 23,  7,  7, 
		15, 32,  0, 38, 
	},

	// after doors palette
	{
/* Not used in Wily stages */
	},

	// actives
	{
	},

	// enemies
	{
		Enemy( 128, 0x00,0x00, 0x0 ),
	},

	// level gfx (id, count) (max 16); id is block*4+bank
	{
		{ 0x40, 8 }, { 0x7A, 1 }, { 0xC2, 4 }, { 0xE2, 1 }, 
		{ 0xD8, 1 }, { 0x5E, 1 }, { 0xC0, 2 }, { 0x43, 4 }, 
		{ 0xD8, 2 }, { 0x63, 8 }, 
	},

	// monster gfx tables (max 11) (5 tiles, 2*3bytepalette)
	{
		{ { 0x7A, 0xC2, 0xC6, 0x62, 0x72, }, { 39,21,15, 48,17,15,} },
		{ { 0x6E, 0x7E, 0xBE, 0x62, 0x72, }, { 48,17,15, 48,21,15,} },
		{ { 0x6E, 0x7E, 0xBE, 0x62, 0x72, }, { 48,21,15, 48,21,15,} },
		{ { 0xD6, 0xB6, 0xBA, 0x76, 0x72, }, { 44,17,15, 48,42,15,} },
		{ { 0xDF, 0xE3, 0xE3, 0xE3, 0xE3, }, { 48,21,15, 48,23,39,} },
	},

	0x00, // Arrival input

	Wily1Ascii, Wily1Char
};


